
<!doctype html>
<script src="../js/all-io.js"></script>
<script src="https://code.jquery.com/jquery-3.2.1.slim.min.js" integrity="sha384-KJ3o2DKtIkvYIK3UENzmM7KCkRr/rE9/Qpg6aAZGJwFDMVNA/GpGFF93hXpG5KkN" crossorigin="anonymous"></script>
<script src="../js/bootstrap.min.js"></script>
<script src="../js/bootstrap-notify.min.js"></script>


<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <meta name="description" content="">
    <meta name="author" content="">

    <title>Gradio</title>

    <!-- Bootstrap core CSS -->
    <link href="../css/bootstrap.min.css" rel="stylesheet">
    <link href="../css/draw-a-digit.css" rel="stylesheet">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
  </head>

  <body>
    <div id="overlay" style="visibility: hidden"></div>
    <nav class="navbar navbar-expand-md navbar-dark bg-dark fixed-top">
      <a class="navbar-brand" href="#">Gradio</a>
      <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarsExampleDefault" aria-controls="navbarsExampleDefault" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>

      <div class="collapse navbar-collapse" id="navbarsExampleDefault">
        <ul class="navbar-nav mr-auto">
        </ul>
          <ul class="navbar-nav">
          <!--<li class="nav-item">-->
            <!--<a class="nav-link" href="#">Help</a>-->
          <!--</li>-->
          <li class="nav-item">
              <em><strong><span id="model-status" style="color:#3ee263">Model: running</strong></em>
          </li>
          </ul>
      </div>
    </nav>
    <main role="main" class="container starter-template">
            <div class="row">
              
<!-- INPUT 
====================================================================================================================================================== -->
           <div id="input"></div>   
<!-- OUTPUT 
====================================================================================================================================================== -->
           <div id="output"></div>   
            

            </div>
      </div>
    </main><!-- /.container -->

<!--     <footer class="footer">
      <div class="container" >
        <span class="text-muted">

            Gradio is open-source, help make it better &nbsp;
            <a href="https://github.com/abidlabs/gradiome" target="_blank" class="fa fa-github"></a>

        </span>

        <span class="text-muted pull-right">

            Found this useful? Kindly spread the word &nbsp;
            <a href="https://www.facebook.com/sharer/sharer.php?u=www.siliconprep.com" target="_blank" class="fa fa-facebook"></a>
            <a href="https://twitter.com/home?status=Check%20out%20Gradio%20tool%20at%20www.siliconprep.com" target="_blank" class="fa fa-twitter"></a>


        </span>
      </div>
    </footer> -->

    <!-- Bootstrap core JavaScript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->

  </body>
</html>







